/*

  version 0.2 - p_e used for sublattice definition
  
   1/4 depleted square lattice
   sublatice A (2*N/3) and B (N/3) with bounds A-A B-B A-B
  0 1 2 3 4 5 6 7 8 9 x
0 A B A B A B A B A B
1   A   A   A   A   A
2 A B A B A B A B A B
3   A   A   A   A   A
4 A B A B A B A B A B
5   A   A   A   A   A
6 A B A B A B A B A B
7   A   A   A   A   A
y

quadratische Gitter (1/4depleted) (use as table?)                v- too large
sites A+B  | N  12  24  48  96 192 384  768 1536 3072 6144 12288 24576 49152
4*N/3 2A=4B| N0 16  32  64 128 256 512 1024 2048 4096 8192 16384 32768 65536
winds y    | N1  4   4   8   8  16  16   32   32   64   64   128   128   256
N0/N1 x    | N2  4   8   8  16  16  32   32   64   64  128   128   256   256
d_x        | N3  0   4   0   8   0  16    0   32    0   64     0   128     0

N1=2^n n>1
N2=N1 or 2*N1

N3=N2-N1
N0=N1*N2
N=3*N0/4

*/
#define N1    4       /* winds n0 % N1 == 0 */
#define N2    8       /* = 4*n0/n1 */
#define N3 (N2-N1)    /* = n2-n1 */
#define N0 (N1*N2)    /* = 4*n0 */
#define N  (3*N0/4)   /* = 3*n0 */

int n0=4, n1=4, n2=8, n3=4, nn=6;

/* get (x,y,z) position of site i (need for graphical output) */ 
int site_pos(int i,int pos[3]){ int x,y;
  /* ---  i -> (x,y)  --- */
  if(2*i<N0){		/* site A */
    y=i/(N2/2);		/* N2=length of winds */
    x=i%(N2/2)*2+y%2;	/* in ungeraden zeilen eine Spalte versetzt */
  } else {		/* site B */
    y=2*((i-N0/2)/(N2/2));
    x=2*((i-N0/2)%(N2/2))+1;
  }
  pos[0]=x; pos[1]=y; pos[2]=0;
  return 0; }

/* Wechselwirkung W=ww[i][j] zwischen site i und j mit J=jj[W] */
/* i -> (x,y) -> neighbour -> (x+dx,y+dy) -> j */

/* j=neighbour(site number) r=bound_type_number */
int ww_neighbour(int i,int n,int *j,int *r){ /* n-th neighbour of site i */
  int x,y,xn,yn; 		

  /* ---  i -> (x,y)  --- */
  if(2*i<N0){		/* site A */
    y=i/(N2/2);		/* N2=length of winds */
    x=i%(N2/2)*2+y%2;	/* in ungeraden zeilen eine Spalte versetzt */
  } else {		/* site B */
    y=2*((i-N0/2)/(N2/2));
    x=2*((i-N0/2)%(N2/2))+1;
  }
  /* printf("%3d -> (%2d %2d)",i,x,y); */
  /* ---  (x,y) -> (x,y)_neighbour --- */
  if(2*i<N0){		/* site A */
    switch(n){
      case 0: *r=1;xn=(x+1   +((y+1)/N1)*N3)%N2;yn=(y+1 )%N1;  break; /* A-A */
      case 1: *r=1;xn=(x-1+N2+((y+1)/N1)*N3)%N2;yn=(y+1 )%N1;  break; /* A-A */
      default: *r=0;*j=i;return -1; }
    if ( y+1>=N1 || abs(xn-x)>1 ) *r=-1;
  } else {		/* site B */
    switch(n){
      case 0: *r=3;xn=(x+1   )%N2;           yn=y; 	    break; /* B-A */
      case 1: *r=3;xn=(x-1+N2)%N2;           yn=y; 	    break; /* B-A */
      case 2: *r=4;xn=(x+((y+1)/N1)*N3)%N2;  yn=(y+1   )%N1;break; /* B-A */
      case 3: *r=4;xn=(x+((y)?0:N2-N3))%N2;  yn=(y-1+N1)%N1;break; /* B-A */
      case 4: *r=2;xn=(x+2   )%N2;           yn=y; 	    break; /* B-B */
      case 5: *r=2;xn=(x+((y+2)/N1)*N3)%N2;  yn=(y+2   )%N1;break; /* B-B */
      default: *r=0;*j=i;return -1; }
    switch(n){
      case 0: if(x+1>=N2) *r=-3; break; /* B-A */
      case 1: if(x-1<  0) *r=-3; break; /* B-A */
      case 2: if(y+1>=N1) *r=-4; break; /* B-A */
      case 3: if(y-1<  0) *r=-4; break; /* B-A */
      case 4: if(x+2>=N2) *r=-2; break; /* B-B */
      case 5: if(y+2>=N1) *r=-2; break; /* B-B */
    }
  }
  x=xn;
  y=yn;
  /* printf(" *%d*",(*r)); */
  /* ---  (x,y) -> i --- */
  if((x+y)%2==0){	/* site A */
    i=(y*N2+x)/2;
  } else if(y%2==0){	/* site B */
    i=N0/2+(N2/2)*(y/2)+x/2;
  } else { printf("error: wrong site (%d %d)\n",x,y);exit(1); }
  /* printf(" -> (%2d %2d) -> %3d",x,y,i);
  if(*r!=ww[oi][i])printf(" error");
  printf("\n"); */
  *j=i; return 0;}

char boundary(int i, int j){
  int pi[3], pj[3];
  site_pos(i, pi);
  site_pos(j, pj);
  return ( ( (pi[0]-pj[0])*(pi[0]-pj[0])
           + (pi[1]-pj[1])*(pi[1]-pj[1])>4 ) ? '+' : ' ' );
}

int main(){
  int ii, jj, i2, r, x, y;
  int pos[3];
  n0=N0/4;   /* num elementar cells */
  n1=N1;     /* winds n0 % N1 == 0 */
  n2= 4*n0/n1;
  n3= n2-n1;
  nn= 3*n0;
  
  printf(
    "# AAB 1/4 depleted square\n"
    "#===base=== 1 0 0 1\n"
    "#===edge=== %d 0 %d %d\n"
    "#===uvec=== 2 0 0 2\n"
    "#===pbcf=== 0 0  %d 0  %d %d  %d %d  0 0\n"
    "#+++model+++\n"
    "# NN Nw Nv\n"
    " %d %d 0\n"
    "#  s X Y Z p_e p_U\n",
      n2, n3, n1,
      n2,  n2+n3, n1,  n3, n1,
      nn, n0*10
  );
  for( ii=0; ii<nn; ii++ ){
    site_pos(ii, pos);
    printf(" %2d %2d %2d  0 %d 7\n",ii,pos[0],pos[1],((ii<2*n0)?8:9));
  }
  printf("# Bond s1 s2 p_t p_J p_V\n"); jj=0;
  for( ii=0;    ii<2*n0; ii++ ){ ww_neighbour(ii,0,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=0;    ii<2*n0; ii++ ){ ww_neighbour(ii,1,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=2*n0; ii<3*n0; ii++ ){ ww_neighbour(ii,0,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=2*n0; ii<3*n0; ii++ ){ ww_neighbour(ii,1,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=2*n0; ii<3*n0; ii++ ){ ww_neighbour(ii,2,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=2*n0; ii<3*n0; ii++ ){ ww_neighbour(ii,3,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=2*n0; ii<3*n0; ii++ ){ ww_neighbour(ii,4,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  for( ii=2*n0; ii<3*n0; ii++ ){ ww_neighbour(ii,5,&i2,&r);
    printf(" %3d %2d %2d 4 %c%d 0\n", jj++,ii,i2,((r<0)?'+':' '),abs(r));
  }
  printf("#---model---\n");
  return 0;
}
